<?php
    
namespace App\Controller;

use App\Templates;
use App\Exception;

use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;

class OrderController {
    
    //const FAILOVER_EMAIL = "aron.rissato@restaurantemadero.com.br"; //depreciado
    const FAILOVER_EMAIL = "delivery@teknisa.com";  
    // Testing URL
    //const TARGET_URL = "https://microservices.dev.rappi.com/api/v2/restaurants-integrations-public-api/"; 
    //const AUTH_URL   = "https://rests-integrations-dev.auth0.com/oauth/token";

    // Url Rappi
    const TARGET_URL = "https://services.rappi.com.br/api/v2/restaurants-integrations-public-api/";
    const AUTH_URL   = "https://rests-integrations.auth0.com/oauth/token";
    
    // const RAP_TOKEN_LIFE = 3600; // one hour
    const RAP_TOKEN_LIFE = 604300; // one week

    private $connection;
    private $util;
    private $branchService;
    private $consumerService;
    private $service;
    private $productService;
    
    private $serverName;
    
    private $mailList;
    
    public function __construct (\Doctrine\DBAL\Connection $connection,
                                    \App\Util\Util $util,
                                    \App\Service\BranchService $branchService,
                                    \App\Service\ConsumerService $consumerService,
                                    \App\Service\OrderService $service,
                                    \App\Service\ProductService $productService) {
                                        
        $this->connection       = $connection;
        $this->util             = $util;
        $this->branchService    = $branchService;
        $this->consumerService  = $consumerService;
        $this->service          = $service;
        $this->productService   = $productService;
        
    }

    public function runWebService (Request $request, Response $response) {
        try {
            
            $req = $request->getParsedBody();
            
            if ($req["auth"] != "81743A8E8D41AF82A4807097CDF4BEC2DDDB9E6334BECCFED4D46FC783A8CA2E") 
                throw new \Exception("Auth inválido.", 666);
            
            $configuration = $this->service->getConfiguration();
            
            if(empty($configuration)) {
                $response->withStatus(200);
                return 0;
            }

            $ufTimezone    = (isset($configuration['IDUFTIMEZONE']) && !empty($configuration['IDUFTIMEZONE'])) ? $configuration['IDUFTIMEZONE'] : "SP";
            date_default_timezone_set($this->util->getCurrentTimezone($ufTimezone));
            
            if($configuration['IDHABINTE'] == 'N') 
                return $response->withStatus(200);
                
            if(empty($configuration['IDTOKENINTGERAL'])) 
                throw new Exception\GeneralErrorException("Credenciais de integração não encontradas");
                
            if(empty($configuration['IDSENHATOKENGERAL'])) 
                throw new Exception\GeneralErrorException("Credenciais de integração não encontradas");
            
            $this->mailList = $this->util->getMailList();
            $configsFile = file_get_contents(__DIR__ . "/../../settings/configs/configs.json");
            $config      = json_decode($configsFile, true);
            $this->serverName = $config['server'];
            if(!empty($configuration['AUTHTKNDTCRIAC']) && ((time() - intval($configuration['AUTHTKNDTCRIAC'])) < self::RAP_TOKEN_LIFE)){
                $this->util->setAccessToken($configuration['AUTHTOKEN']);
                
            } else{
                $this->util->authorizeAccess(self::AUTH_URL, $configuration, $config['audience']);
                $newToken = $this->util->getAccessToken();
                $this->service->updateAccessToken($newToken);
            }

            $orders = [];
            
            $unprocessedOrders = json_decode($this->util->executeWebService(self::TARGET_URL."orders/status/sent", "GET"));
            $unfilteredOrders = json_decode($this->util->executeWebService(self::TARGET_URL."orders", "GET"));
	        $dateLog = date("d/m/Y H:i:s");
	        $receivedOrders = "$dateLog - Pedido(s) recebido(s): ";
            
            if (!empty($unprocessedOrders)){
                
                foreach ($unprocessedOrders as $pendingOrder) {
                    $sentOrder = json_decode(json_encode($pendingOrder), true);
                    $this->util->writeLog("execution","", $dateLog . " Pedido #" . $sentOrder["order_detail"]["order_id"] . " - Loja Id: " . $sentOrder["store"]["internal_id"] . " - Pendente de processamento. \n");

                    if (!empty($unfilteredOrders)){
                        foreach ($unfilteredOrders as $order){
                            $newOrder = json_decode(json_encode($order), true);
                            $receivedOrders .= $newOrder['order_detail']['order_id'] . ", ";
                            
                            if ($sentOrder["order_detail"]["order_id"] == $newOrder["order_detail"]["order_id"] && $sentOrder["store"]["internal_id"] == $newOrder["store"]["internal_id"]) {
                                $validatePendingOrder = true;
                            }
                        }
                    }
                    if (!isset($validatePendingOrder)) {
                        array_push($orders, json_decode(json_encode($pendingOrder), true));
		                $this->util->writeLog("execution","", date("d/m/Y H:i:s") . " Pedido #" . $sentOrder["order_detail"]["order_id"] . " - Loja Id: " . $sentOrder["store"]["internal_id"] . " - Adicionado a lista para nova tentativa de processamento. \n");
                    }
                }
            }

            foreach ($unfilteredOrders as $order) {
                $actualOrder = json_decode(json_encode($order), true);
                array_push($orders, json_decode(json_encode($order), true));
                $receivedOrders .= $actualOrder['order_detail']['order_id'] . ", ";
            }
            
            $receivedOrders = substr($receivedOrders, 0, -2);
            $receivedOrders .= "\n";

            if (empty($orders)) {
                return $response->withStatus(200);
            }
            else {
                $this->util->writeLog("execution", "", $receivedOrders);
            }
            
            foreach ($orders as $order) {
                
                $this->connection->beginTransaction();
                $branchInfo = $this->fetchBranch($order["store"]["internal_id"]);
                $filename = (isset($order['order_detail']['order_id']) ? "{$branchInfo['CDFILIAL']}-{$order['order_detail']['order_id']}" : ("INDEFINIDO-" . date("H-i-s"))) . ".json"; 
                
                try {
                    $this->util->writeLog("order", $filename, $order);
                    
                    $comanda = $this->processOrder($order, $branchInfo["CDFILIAL"]);
                    $this->service->validateOrder($comanda['CDFILIAL'], $comanda['NRVENDAREST']);
                    
                    $this->util->executeWebService(self::TARGET_URL."orders/{$order['order_detail']['order_id']}/take/", "PUT");
                    
                    $this->connection->commit();
                    
                    $dateLog = date("d/m/Y H:i:s");
                    $content = "[$dateLog - Pedido Processado]: #{$order['order_detail']['order_id']} - {Filial: $comanda[CDFILIAL] / Número da venda: $comanda[NRVENDAREST] / Número da comanda: $comanda[NRCOMANDA] }\n";
                    
                    $this->util->writeLog('order_processed', "", $content);
                    $this->util->writeLog("execution", "", "$dateLog Pedido #" . $order['order_detail']['order_id'] . " filial " . $comanda['CDFILIAL'] . " integrado.\n");
                    
                } catch (Exception\NetworkException $e) {
                    $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro de Rede (NetworkException): {$e->getMessage()}\n");
                    
                    $this->connection->rollback();
                    
                    if (isset($order['order_detail']['order_id'])) {
                        
                        $reason = 'Nao foi possivel integrar o pedido: ' . $e->getMessage();
                        $reason = strtoupper($this->util->removeAcentos($reason));
                        
                        $this->util->executeWebService(self::TARGET_URL . "orders/{$order['order_detail']['order_id']}/reject", "PUT", json_encode([
                            "reason"   => $reason
                        ]));
                    }
                } catch (Exception\IntegrationException $e) {
                    $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Error: " . $e->getMessage() . "\n");
                    $this->connection->rollback();
                    
                    if(!isset($branchInfo["CDFILIAL"])) {
                        $branchInfo["CDFILIAL"] = 'undefined';
                    }
                    
                    $dateLog = date("d/m/Y H:i:s");
                    $content = "[IntegrationException - $dateLog]: Filial -> [$branchInfo[CDFILIAL]]; codPedido -> #{$order['order_detail']['order_id']}; Error -> {$e->getMessage()}\n";
                    
                    $this->util->writeLog('order_error', $filename, $content);
                    
                    if (isset($order['order_detail']['order_id'])) {
                       
                        $reason = 'Nao foi possivel integrar o pedido: ' . $e->getMessage();
                        $reason = strtoupper($this->util->removeAcentos($reason));
                        
                        $this->util->executeWebService(self::TARGET_URL . "orders/{$order['order_detail']['order_id']}/reject", "PUT", json_encode([
                            "reason"   => $reason 
                        ]));
                    }
                } catch (Exception\ParamsException $e) {
                    $this->connection->rollback();
                    
                    if(!isset($branchInfo["CDFILIAL"])) {
                        $branchInfo["CDFILIAL"] = 'undefined';
                    }
                    
                    $dateLog = date("d/m/Y H:i:s");
                    $content = "[ParamsException - $dateLog]: Filial -> [$branchInfo[CDFILIAL]]; codPedido -> #{$order['order_detail']['order_id']}; Error -> {$e->getMessage()}\n";
                    
                    $this->util->writeLog('order_error', $filename, $content);
                    
                    $reason = 'Nao foi possivel integrar o pedido: ' . $e->getMessage();
                    $reason = strtoupper($this->util->removeAcentos($reason));
                    
                    $this->util->executeWebService(self::TARGET_URL . "orders/{$order['order_detail']['order_id']}/reject", "PUT", json_encode([
                        "reason" => $reason
                    ]));
                    
                    $template = file_get_contents(__DIR__ ."/../../templates/error.html");
                    $this->util->sendMail((!empty($this->mailList) ? $this->mailList : [self::FAILOVER_EMAIL]), [], "Integração Teknisa x Rappi", $template, [
                        ":INTEGRACAO"    => "Rappi",
                        ":NUMERO_PEDIDO" => "#".$branchInfo['CDFILIAL']."-".$order["order_detail"]["order_id"],
                        ":NOME_SERVIDOR" => $this->serverName,
                        ":ERRO_MSG"      => $e->getMessage(),
                        ":JSON_PEDIDO"   => json_encode($order, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE),
                        ":GERADO_EM"     => date("d/m/Y H:i:s")
                    ]);
                } catch (\Exception $e) {
                    $this->connection->rollback();
                    
                    if(!isset($branchInfo["CDFILIAL"])) {
                        $branchInfo["CDFILIAL"] = 'undefined';
                    }
                    
                    $dateLog = date("d/m/Y H:i:s");
                    $content = "[ErrorLog - $dateLog]: Filial -> [$branchInfo[CDFILIAL]]; codPedido -> #{$order['order_detail']['order_id']}; Error -> {$e->getMessage()}\n";
                    
                    $this->util->writeLog('order_error', $filename, $content);
                     
                    $reason = 'Nao foi possivel integrar o pedido: ' . $e->getMessage();
                    $reason = strtoupper($this->util->removeAcentos($reason));
                    
                    $this->util->executeWebService(self::TARGET_URL . "orders/{$order['order_detail']['order_id']}/reject", "PUT", json_encode([
                        "reason"   => $reason
                    ]));
                    
                    $template = file_get_contents(__DIR__ ."/../../templates/error.html");
                    $this->util->sendMail((!empty($this->mailList) ? $this->mailList : [self::FAILOVER_EMAIL]), [], "Integração Teknisa x Rappi", $template, [
                        ":INTEGRACAO"    => "Rappi",
                        ":NUMERO_PEDIDO" => "#".$branchInfo['CDFILIAL']."-".$order['order_detail']['order_id'],
                        ":NOME_SERVIDOR" => $this->serverName,
                        ":ERRO_MSG"      => $e->getMessage(),
                        ":JSON_PEDIDO"   => json_encode($order, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE),
                        ":GERADO_EM"     => date("d/m/Y H:i:s")
                    ]);
                }
            }
            
        } catch (Exception\NetworkException $e) {
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro de Rede (NetworkException): {$e->getMessage()}\n");
        } catch (Exception\GeneralErrorException $e) {
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro Geral (GeneralErrorException): {$e->getMessage()}\n");
            $this->service->disableIntegration();
            
            $template = file_get_contents(__DIR__ ."/../../templates/general-error.html");
            $this->util->sendMail((!empty($this->mailList) ? $this->mailList : [self::FAILOVER_EMAIL]), [], "Integração Teknisa x Rappi", $template, [
                ":INTEGRACAO"    => "Rappi",
                ":NOME_SERVIDOR" => $this->serverName,
                ":ERRO_MSG"      => $e->getMessage(),
                ":GERADO_EM"     => date("d/m/Y H:i:s")
            ]);
        } catch (\Exception $e) {
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro Desconhecido: {$e->getMessage()}\n");
        }
        
        return $response->withStatus(200);
    }
    
    public function welcome(Request $request, Response $response) {
        $sql        = "SELECT CDFILIAL FROM FILIAL";
        $result     = $this->connection->fetchAll($sql);

        $versionInfo = $this->getVersionInfo();

        return $response->withJson([
            'version'     => $versionInfo['version'],
            'revision'    => $versionInfo['revision'],
            'date'        => $versionInfo['date'],
            'branchs'     => count($result)
            //'database'    => $this->connection->getParams()
        ]);
    }

    private function getVersionInfo() {
        $file = file_get_contents(__DIR__ . "/../../version.json");
        return json_decode($file, true);
    }
    
    private function fetchBranch($storeId) {
        return $this->branchService->fetchBranch($storeId);
    }
    
    private function processOrder($order, $branch) {
        $generalParams = $this->service->getDefaultParams($branch);
        $deliveryParams = $this->service->getDeliveryMethodParams($branch);
        
        $generalParams["CPF"] = "";
        $generalParams["NRCONTATOSAC"] = "";
        
        if(isset($deliveryParams["IDUTLVNDPDRPALC"]) && ($deliveryParams["IDUTLVNDPDRPALC"] == "N")) {
            if(empty($deliveryParams['CDLOJA'])) {
                throw new Exception\ParamsException("Não existe um código de loja associado ao restaurante <b>{$order['store']['id']}</b> Rappi.");
            }

            if(empty($deliveryParams['CDCAIXA'])) {
                throw new Exception\ParamsException("Não existe um código de caixa associado ao restaurante <b>{$order['store']['id']}</b> Rappi.");
            }
            
            $generalParams["CDLOJA"] = $deliveryParams["CDLOJA"];
            $generalParams["CDCAIXA"] = $deliveryParams["CDCAIXA"];
        }

        if(!empty($deliveryParams['NRCONTATOSAC'])) {
            $generalParams["NRCONTATOSAC"] = $deliveryParams['NRCONTATOSAC'];
        }
        
        if($order['order_detail']['delivery_method'] === 'marketplace') {
            $consumer = $this->consumerService->getDefaultConsumer($generalParams["CDCLIENTE"], $generalParams["CDCONSUMIDOR"]);
            $consumer['SGESTADO']        = $order['order_detail']['delivery_information']['federal_unit'];
            $consumer['NMMUNICIPIO']     = strtoupper($this->util->removeAcentos($order['order_detail']['delivery_information']['city']));
            $consumer['NRCEPCONS']       = str_replace("-", "", $order['order_detail']['delivery_information']['postal_code']);
            $consumer['NMBAIRCONS']      = strtoupper($this->util->removeAcentos($order['order_detail']['delivery_information']['neighborhood']));
            $consumer['NRENDECONS']      = $order['order_detail']['delivery_information']['street_number'];
            $consumer['DSENDECONS']      = strtoupper($this->util->removeAcentos($order['order_detail']['delivery_information']['street_name']));
            $consumer['DSCOMPLENDECONS'] = strtoupper($this->util->removeAcentos($order['order_detail']['delivery_information']['street_shorcut']).' '.strtoupper($order['order_detail']['delivery_information']['complement']));
            $consumer['CDMUNICIPIO']     = $this->consumerService->getConsumerCity($consumer);
            $consumer['CDBAIRRO']        = $this->consumerService->getConsumerBairCode($consumer);
            $consumer['DSREFERENDECONS'] = '';
        } else{
            $consumer = $this->consumerService->getDefaultConsumer($generalParams["CDCLIENTE"], $generalParams["CDCONSUMIDOR"]);
            if(empty($consumer['SGESTADO'])){
                throw new Exception\GeneralErrorException("Cadastro de endereço do consumidor padrão incompleto. Estado não encontrado.");
            } else if(empty($consumer['CDMUNICIPIO'])){
                throw new Exception\GeneralErrorException("Cadastro de endereço do consumidor padrão incompleto. Municipio não encontrado.");
            } else if(empty($consumer['NRCEPCONS'])){
                throw new Exception\GeneralErrorException("Cadastro de endereço do consumidor padrão incompleto. CEP não encontrado.");
            } else if(empty($consumer['NMBAIRCONS'])){
                //throw new Exception\GeneralErrorException("Cadastro de endereço do consumidor padrão incompleto. Bairro não encontrado.");
            } else if(empty($consumer['NRENDECONS'])){
                throw new Exception\GeneralErrorException("Cadastro de endereço do consumidor padrão incompleto. Numero não encontrado.");
            }
        }
        
        if(empty($consumer['CDMUNICIPIO'])){
            $endeFili = $this->branchService->getBranchCityCode($branch);
            $consumer['CDMUNICIPIO'] = $endeFili['CDMUNICIPIO'];
            $consumer['SGESTADO'] = $endeFili['SGESTADO'];
            $consumer['CDBAIRRO'] = null;
        }
        
        $consumer["consumerInfo"] = $order["customer"];
        
        return $this->service->openOrderBill($generalParams, $order["order_detail"], $consumer);
    }
    
    public function debugOrder (Request $request, Response $response) {
        
        try {
            $req = $request->getParsedBody();

            $ufTimezone     = "SP";
            
            date_default_timezone_set($this->util->getCurrentTimezone($ufTimezone));

            $order          = $req["data"];

            $this->connection->beginTransaction();

            try {
                
                $branchInfo = $this->fetchBranch($order["store"]["internal_id"]);

                $filename   = (isset($order['order_detail']['order_id']) ? "{$branchInfo['CDFILIAL']}-{$order['order_detail']['order_id']}" : ("INDEFINIDO-" . date("H-i-s"))) . ".json"; 

                $this->util->writeLog("order", $filename, $order);
                
                $comanda    = $this->processOrder($order, $branchInfo["CDFILIAL"]);

                $this->service->validateOrder($comanda['CDFILIAL'], $comanda['NRVENDAREST']);
                
                $this->connection->commit();
                
                $dateLog    = date("d/m/Y H:i:s");
                $content    = "[$dateLog - Pedido Processado]: #{$order['order_detail']['order_id']} - {Filial: $comanda[CDFILIAL] / Número da venda: $comanda[NRVENDAREST] / Número da comanda: $comanda[NRCOMANDA] }\n";
                
                $this->util->writeLog('order_processed', "", $content);
                $this->util->writeLog("execution", "", "$dateLog Pedido #" . $order['order_detail']['order_id'] . " filial " . $comanda['CDFILIAL'] . " integrado.\n");
                echo "Pedido integrado! AEEEEE \o/ \n";
                
            } catch (Exception\NetworkException $e) {
                $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro de Rede (NetworkException): {$e->getMessage()}\n");
                
                $this->connection->rollback();
                
            } catch (Exception\IntegrationException $e) {
                $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Error: " . $e->getMessage() . "\n");
                $this->connection->rollback();
                
                if(!isset($branchInfo["CDFILIAL"])) {
                    $branchInfo["CDFILIAL"] = 'undefined';
                }
                
                $dateLog = date("d/m/Y H:i:s");
                $content = "[IntegrationException - $dateLog]: Filial -> [$branchInfo[CDFILIAL]]; codPedido -> #{$order['order_detail']['order_id']}; Error -> {$e->getMessage()}\n";
                
                $this->util->writeLog('order_error', $filename, $content);
                
            } catch (Exception\ParamsException $e) {
                $this->connection->rollback();
                
                if(!isset($branchInfo["CDFILIAL"])) {
                    $branchInfo["CDFILIAL"] = 'undefined';
                }
                
                $dateLog = date("d/m/Y H:i:s");
                $content = "[ParamsException - $dateLog]: Filial -> [$branchInfo[CDFILIAL]]; codPedido -> #{$order['order_detail']['order_id']}; Error -> {$e->getMessage()}\n";
                
                $this->util->writeLog('order_error', $filename, $content);
                
            } catch (\Exception $e) {
                $this->connection->rollback();
                
                if(!isset($branchInfo["CDFILIAL"])) {
                    $branchInfo["CDFILIAL"] = 'undefined';
                }
                
                $dateLog = date("d/m/Y H:i:s");
                $content = "[ErrorLog - $dateLog]: Filial -> [$branchInfo[CDFILIAL]]; codPedido -> #{$order['order_detail']['order_id']}; Error -> {$e->getMessage()}\n";
                
                $this->util->writeLog('order_error', $filename, $content);
                    
            }
            
        } catch (Exception\NetworkException $e) {
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro de Rede (NetworkException): {$e->getMessage()}\n");
        } catch (Exception\GeneralErrorException $e) {
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro Geral (GeneralErrorException): {$e->getMessage()}\n");
            $this->service->disableIntegration();
            
        } catch (\Exception $e) {
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro Desconhecido: {$e->getMessage()}\n");
        }
        
        return $response->withStatus(200);        

    }
}
