<?php

namespace App\Factory;

use App\Exception;

class BranchFactory {
    private $connection;

    public function __construct (\Doctrine\DBAL\Connection $connection) {
        $this->connection = $connection;
    }
    
    public function fetchBranch ($storeId) {
        $sql = "
            SELECT CDFILIAL FROM INTEGRACAOFILI IF 
            JOIN INTEGRACAO I
            ON IF.CDINTEGRACAO = I.CDINTEGRACAO
            WHERE I.IDINTEGRACAO = 'RAP'
                AND IF.IDTOKEN   = :CDFILIALEXT
        ";

        $params = [
            ":CDFILIALEXT" => $storeId
        ];
        
        $branchInfo = $this->connection->fetchAssoc($sql, $params);
        
        if(empty($branchInfo)) {
            throw new Exception\ParamsException("Não existe filial associada ao Identificador: $storeId (id do restaurante no Rappi)", 101);
        }
        
        return $branchInfo;
    }
    
    public function getDefaultParams($cdfilial) {
        $sql = "
            SELECT PV.CDFILIAL, PV.CDVENDEDOR, V.CDOPERADOR, V.CDCAIXA, C.CDLOJA, PV.CDCLIENTE,
                   NVL(PV.CDCONSUMIDOR, PV.CDCONSUMIDPADRAO) CDCONSUMIDOR, L.NRMESAPADRAO AS NRMESA, V.NMFANVEN, L.IDCONTROPROD, ED.SGESTADO
              FROM PARAVEND PV
         LEFT JOIN VENDEDOR V
                ON V.CDVENDEDOR = PV.CDVENDEDOR
         LEFT JOIN CAIXA C
                ON C.CDCAIXA = V.CDCAIXA
         LEFT JOIN LOJA L
                ON L.CDLOJA = C.CDLOJA
               AND L.CDFILIAL = PV.CDFILIAL
         LEFT JOIN ENDEFILI ED
                ON ED.CDFILIAL = PV.CDFILIAL
             WHERE PV.CDFILIAL = :CDFILIAL
            ORDER BY NRMESA, IDCONTROPROD, CDCONSUMIDOR, CDCLIENTE, CDLOJA, CDCAIXA, CDOPERADOR, CDVENDEDOR
        ";

        $params = [
            'CDFILIAL' => $cdfilial
        ];

        return $this->connection->fetchAssoc($sql, $params);
    }
    
    public function getBranchCityCode($branchId) {
        $sql = "
            SELECT CDMUNICIPIO, SGESTADO
              FROM ENDEFILI
             WHERE CDFILIAL = :CDFILIAL
        ";

        $params = [
            ":CDFILIAL" => $branchId
        ];
        
        return $this->connection->fetchAll($sql, $params)[0];
    }
   
    public function getDeliveryMethodParams($cdFilial) {
        $sql = "
            SELECT IF.IDUTLENTRETERC, IF.IDRETTXENTR, I.IDUTLVNDPDRPALC, IF.NRCONTATOSAC, IF.CDLOJA, IF.CDCAIXA
              FROM INTEGRACAOFILI IF
              JOIN INTEGRACAO I
                ON IF.CDINTEGRACAO = I.CDINTEGRACAO
             WHERE IF.CDFILIAL = :CDFILIAL
               AND I.IDINTEGRACAO = 'RAP'
        ";

        $params = [
            ":CDFILIAL" => $cdFilial
        ];

        return $this->connection->fetchAssoc($sql, $params);
        
    }
}
