<?php

namespace App\Service;

use App\Exception;

class ProductService {
    
    private $util;
    private $factory;
    
    public function __construct (\App\Util\Util $util, 
                                 \App\Factory\ProductFactory $factory) {

        $this->util    = $util;
        $this->factory = $factory;
    }
    
    public function processProduct ($generalParams, $product, $nrvendarest, $nrcomanda, &$ITCOMANDAVEN_SEQ, $isSku) {
        switch($product["type"]) {
            case "product":
                $processedProduct = $this->processSimpleProduct($generalParams, $product, $nrvendarest, $nrcomanda, $ITCOMANDAVEN_SEQ, $isSku);
                break;
            
            default: 
                throw new \Exception("O produto {$product['name']} (modelo {$product['type']}) não é suportado pela integração, verifique os modelos de produtos suportados no manual da integração.");
                break;
        }

        return $processedProduct;
    }
    
    private function processSimpleProduct ($generalParams, $product, $nrvendarest, $nrcomanda, &$ITCOMANDAVEN_SEQ, $isSku) {
        $productItems = [];
        if($isSku){
            $productDetails = $this->factory->getProduct($product["sku"]);
            if (empty($productDetails)) 
                throw new Exception\ParamsException("O produto {$product['name']} ({$product['sku']}) não encontrado. Verifique o código cadastrado no portal rappi.");
        } else{
            $productDetails = $this->factory->getProductMap($product["id"]);
            if (empty($productDetails)) 
                throw new Exception\ParamsException("O produto {$product['name']} ({$product['id']}) não está mapeado com o código de produto Teknisa.");
        }

        $productDetails = array_merge($productDetails, $product);
        
        for($i = 0; $i < $product["quantity"]; $i++) {
            $nrSeqProdComPai = str_pad((string) ($ITCOMANDAVEN_SEQ + 1), 6, '0', STR_PAD_LEFT);
            $ITCOMANDAVEN_SEQ++;

            $productDetails["comments"] = (isset($productDetails["comments"]) && !empty($productDetails["comments"])) ? (" {$productDetails['comments']}; ") : "";
            
            $pruductSimpleTemplate = [
                "CDFILIAL"         => $generalParams["CDFILIAL"],
                "NRVENDAREST"      => $nrvendarest, 
                "NRCOMANDA"        => $nrcomanda,
                "NRPRODCOMVEN"     => $nrSeqProdComPai,
                "CDPRODUTO"        => $productDetails["CDPRODUTO"],
                "QTPRODCOMVEN"     => 1,
                "VRPRECCOMVEN"     => $productDetails["price"], 
                "TXPRODCOMVEN"     => $productDetails["comments"],
                "IDSTPRCOMVEN"     => $generalParams["IDCONTROPROD"] === "S" ? "1" : "3",
                "VRDESCCOMVEN"     => !empty($productDetails["discount"]) ? $productDetails["discount"] : 0,
                "NRLUGARMESA"      => "01",
                "IDPRODIMPFIS"     => "N",
                "CDLOJA"           => $generalParams["CDLOJA"],
                "VRACRCOMVEN"      => 0,
                "NRSEQPRODCOM"     => null,
                "NRSEQPRODCUP"     => $generalParams["NRSEQPRODCUP"],
                "VRPRECCLCOMVEN"   => 0,
                "CDCAIXACOLETOR"   => $generalParams["CDCAIXA"],
                "CDPRODPROMOCAO"   => null,
                "CDVENDEDOR"       => $generalParams["CDVENDEDOR"],
                "NRINSCRCONS"      => preg_replace("/[^0-9]/", "", $generalParams["CPF"]),
                "DSOBSPEDDIGCMD"   => null,
                "ITEM_OBSERVATION" => []
            ];

            foreach ($product["subitems"] as $item) {
                if($isSku){
                    if($item['price'] >= 0.01){
                        if (strlen($item['sku']) === 5 && substr($item['sku'], 2, 1) === '|'){
                            $observationDetails = $this->factory->getOcorencia($item["sku"], true);
                        } else{
                            $observationDetails = $this->factory->getProduct($item["sku"]);
                        }
                        
                        if (empty($observationDetails))
                            throw new Exception\ParamsException("Não foi possivel encontrar o complemento de código: {$item['sku']}");
                            
                        for($y = 0; $y < $item['quantity']; $y++){
                            $observationDetails["NRPRODCOMVEN"] = str_pad((string) ($ITCOMANDAVEN_SEQ + 1), 6, '0', STR_PAD_LEFT);
                            $ITCOMANDAVEN_SEQ++;
                            
                            $obsReturn = array(
                                "CDFILIAL"       => $generalParams["CDFILIAL"],
                                "NRVENDAREST"    => $nrvendarest,
                                "NRCOMANDA"      => $nrcomanda,
                                "NRPRODCOMVEN"   => $observationDetails["NRPRODCOMVEN"],
                                "CDPRODUTO"      => $observationDetails["CDPRODUTO"],
                                "QTPRODCOMVEN"   => 1,
                                "VRPRECCOMVEN"   => $item['price'], 
                                "TXPRODCOMVEN"   => " ",
                                "IDSTPRCOMVEN"   => $generalParams["IDCONTROPROD"] === "S" ? "1" : "4",
                                "VRDESCCOMVEN"   => 0,
                                "NRLUGARMESA"    => "01",
                                "IDPRODIMPFIS"   => "N",
                                "CDLOJA"         => $generalParams["CDLOJA"],
                                "VRACRCOMVEN"    => 0,
                                "NRSEQPRODCOM"   => $this->util->formatNrSeqProdCom($nrSeqProdComPai),
                                "NRSEQPRODCUP"   => $generalParams["NRSEQPRODCUP"],
                                "VRPRECCLCOMVEN" => 0,
                                "CDCAIXACOLETOR" => $generalParams["CDCAIXA"],
                                "CDPRODPROMOCAO" => null,
                                "CDVENDEDOR"     => $generalParams["CDVENDEDOR"],
                                "NRINSCRCONS"    => preg_replace("/[^0-9]/", "", $generalParams["CPF"]),
                                "DSOBSPEDDIGCMD" => "",
                                
                            );
                            
                            array_push($productItems, $obsReturn); 
                        }
                        
                        if (strlen($item['sku']) === 5 && substr($item['sku'], 2, 1) === '|'){
                            array_push($pruductSimpleTemplate['ITEM_OBSERVATION'], [
                                'CDFILIAL'        => $generalParams["CDFILIAL"],  
                                'NRVENDAREST'     => $nrvendarest, 
                                'NRCOMANDA'       => $nrcomanda, 
                                'NRPRODCOMVEN'    => $nrSeqProdComPai,  
                                'CDGRPOCOR'       => $observationDetails['CDGRPOCOR'],   
                                'CDOCORR'         => $observationDetails['CDOCORR'],  
                                'NRPRODCOMVENOBS' => null, //$observationDetails["NRPRODCOMVEN"],  
                                'QTPRODCOMVENOBS' => $item['quantity']
                            ]);   
                        }
                    }else{
                        $observationDetails = $this->factory->getOcorencia($item["sku"]);
                        
                        if (empty($observationDetails))
                            throw new Exception\ParamsException("Não foi possivel encontrar o complemento de código: {$item['sku']}");
                            
                        array_push($pruductSimpleTemplate['ITEM_OBSERVATION'], [
                            'CDFILIAL'        => $generalParams["CDFILIAL"],  
                            'NRVENDAREST'     => $nrvendarest, 
                            'NRCOMANDA'       => $nrcomanda, 
                            'NRPRODCOMVEN'    => $nrSeqProdComPai,  
                            'CDGRPOCOR'       => $observationDetails['CDGRPOCOR'],   
                            'CDOCORR'         => $observationDetails['CDOCORR'],  
                            'NRPRODCOMVENOBS' => null,  
                            'QTPRODCOMVENOBS' => 1
                        ]);
                    }
                } else{
                    if($item['price'] >= 0.01) {
                        $validateProd = $this->factory->getProductMap($item["id"]);
                        
                        if (empty($validateProd)) 
                            throw new Exception\ParamsException("O produto {$item['name']} ({$item['id']}) não está mapeado com o código de produto Teknisa.");
                        
                        for($y = 0; $y < $item["quantity"]; $y++) {
                            $itemDetails = array_merge($validateProd, $item);
        
                            $itemDetails["NRPRODCOMVEN"] = str_pad((string) ($ITCOMANDAVEN_SEQ + 1), 6, '0', STR_PAD_LEFT);
                            $ITCOMANDAVEN_SEQ++;
                            
                            $itemReturn = [
                                "CDFILIAL"          => $generalParams["CDFILIAL"],
                                "NRVENDAREST"       => $nrvendarest, 
                                "NRCOMANDA"         => $nrcomanda,
                                "NRPRODCOMVEN"      => $itemDetails["NRPRODCOMVEN"],
                                "CDPRODUTO"         => $itemDetails["CDPRODUTO"],
                                "QTPRODCOMVEN"      => 1,
                                "VRPRECCOMVEN"      => $itemDetails["price"],
                                "TXPRODCOMVEN"      => "",
                                "IDSTPRCOMVEN"      => $generalParams["IDCONTROPROD"] === "S" ? "1" : "3",
                                "VRDESCCOMVEN"      => !empty($item["discount"]) ? $item["discount"] : 0,
                                "NRLUGARMESA"       => "01",
                                "IDPRODIMPFIS"      => "N",
                                "CDLOJA"            => $generalParams["CDLOJA"],
                                "VRACRCOMVEN"       => 0,
                                "NRSEQPRODCOM"      => $this->util->formatNrSeqProdCom($nrSeqProdComPai),
                                "NRSEQPRODCUP"      => $generalParams["NRSEQPRODCUP"],
                                "VRPRECCLCOMVEN"    => 0,
                                "CDCAIXACOLETOR"    => $generalParams["CDCAIXA"],
                                "CDPRODPROMOCAO"    => null,
                                "CDVENDEDOR"        => $generalParams["CDVENDEDOR"],
                                "NRINSCRCONS"       => preg_replace("/[^0-9]/", "", $generalParams["CPF"]),
                                "DSOBSPEDDIGCMD"    => null,
                                "ITEM_OBSERVATION"  => [] //
                            ];
        
                            array_push($productItems, $itemReturn);
                        }
    
                    } else {
    
                        $mapProd      = $this->factory->getProductMap($item["id"]);
                        $validateProd = $this->factory->getObservation($mapProd['CDPRODUTO'], $generalParams["CDFILIAL"], $generalParams["CDLOJA"]);
    
                        if(empty($pruductSimpleTemplate['TXPRODCOMVEN'])){
                            $pruductSimpleTemplate['TXPRODCOMVEN'] = ' ';
                        }
                        
                        if(empty($validateProd)) {
                            $pruductSimpleTemplate['TXPRODCOMVEN'] .= strtoupper("{$item['name']}; ");
                        } else {
                            $pruductSimpleTemplate['TXPRODCOMVEN'] .= strtoupper("{$validateProd['DSOCORR']}; ");
                            array_push($pruductSimpleTemplate['ITEM_OBSERVATION'], [
                                'CDFILIAL'        => $generalParams["CDFILIAL"],
                                'NRVENDAREST'     => $nrvendarest,
                                'NRCOMANDA'       => $nrcomanda,
                                'NRPRODCOMVEN'    => $nrSeqProdComPai,
                                'CDGRPOCOR'       => $validateProd['CDGRPOCOR'],
                                'CDOCORR'         => $validateProd['CDOCORR'],
                                'NRPRODCOMVENOBS' => null,
                                'QTPRODCOMVENOBS' => $item['quantity']
                            ]);
                        }
                    }
                    }
            }
            
            array_push($productItems, $pruductSimpleTemplate);
            
        }
        return $productItems;
    }

    public function insertProduct ($product) {
        $this->factory->insertProduct($product);
        
        if(!empty($product["ITEM_OBSERVATION"])) {
            foreach($product["ITEM_OBSERVATION"] as $obsItem) {
                $this->factory->insertObservation($obsItem);
            }
        }
    }
    
}
